__version__ = "0.1.104"
__long_desc__ = """
# swarmaURI sdk

This repository includes core interfaces, standard ABCs, and standard concrete references of the SwarmaURI Framework.


## Core 
- ABCs
- Utilities

## Standard
- Concrete Classes

## Community
- Concrete Classes that utilize third party plug-ins

## Experimental
- Tools in development


"""